@@ -13,6 +13,7 @@ Module for ANDES models.
 import logging
 import pprint
 from collections import OrderedDict
+from textwrap import wrap
 from typing import Callable, Iterable, Union
 
 import numpy as np
@@ -44,62 +45,69 @@ class Model:
 
 
     After subclassing `ModelData`, subclass `Model`` to complete a DAE model.
-    Subclasses of `Model` defines DAE variables, services, and other types of parameters,
-    in the constructor ``__init__``.
+    Subclasses of `Model` define DAE variables, services, and other types of
+    parameters, in the constructor ``__init__``.
 
     Attributes
     ----------
     num_params : OrderedDict
-        {name: instance} of numerical parameters, including internal
-        and external ones
+        {name: instance} of numerical parameters, including internal and
+        external ones
 
     Examples
     --------
-    Take the static PQ as an example, the subclass of `Model`, `PQ`, should looks like ::
+    Take the static PQ as an example, the subclass of `Model`, `PQ`, should look
+    like ::
 
         class PQ(PQData, Model):
             def __init__(self, system, config):
-                PQData.__init__(self)
-                Model.__init__(self, system, config)
+                PQData.__init__(self) Model.__init__(self, system, config)
 
-    Since `PQ` is calling the base class constructors, it is meant to be the final class and not further
-    derived.
-    It inherits from `PQData` and `Model` and must call constructors in the order of `PQData` and `Model`.
-    If the derived class of `Model` needs to be further derived, it should only derive from `Model`
-    and use a name ending with `Base`. See :py:class:`andes.models.synchronous.GENBASE`.
+    Since `PQ` is calling the base class constructors, it is meant to be the
+    final class and not further derived. It inherits from `PQData` and `Model`
+    and must call constructors in the order of `PQData` and `Model`. If the
+    derived class of `Model` needs to be further derived, it should only derive
+    from `Model` and use a name ending with `Base`. See
+    :py:class:`andes.models.synchronous.genbase.GENBase`.
 
-    Next, in `PQ.__init__`, set proper flags to indicate the routines in which the model will be used ::
+    Next, in `PQ.__init__`, set proper flags to indicate the routines in which
+    the model will be used ::
 
         self.flags.update({'pflow': True})
 
-    Currently, flags `pflow` and `tds` are supported. Both are `False` by default, meaning the model is
-    neither used in power flow nor time-domain simulation. **A very common pitfall is forgetting to set the flag**.
+    Currently, flags `pflow` and `tds` are supported. Both are `False` by
+    default, meaning the model is neither used in power flow nor in time-domain
+    simulation. **A very common pitfall is forgetting to set the flag**.
 
-    Next, the group name can be provided. A group is a collection of models with common parameters and variables.
-    Devices idx of all models in the same group must be unique. To provide a group name, use ::
+    Next, the group name can be provided. A group is a collection of models with
+    common parameters and variables. Devices' idx of all models in the same
+    group must be unique. To provide a group name, use ::
 
         self.group = 'StaticLoad'
 
-    The group name must be an existing class name in :py:mod:`andes.models.group`.
-    The model will be added to the specified group and subject to the variable and parameter policy of the
-    group.
-    If not provided with a group class name, the model will be placed in the `Undefined` group.
+    The group name must be an existing class name in
+    :py:mod:`andes.models.group`. The model will be added to the specified group
+    and subject to the variable and parameter policy of the group. If not
+    provided with a group class name, the model will be placed in the
+    `Undefined` group.
 
-    Next, additional configuration flags can be added.
-    Configuration flags for models are load-time variables specifying the behavior of a model.
-    It can be exported to an `andes.rc` file and automatically loaded when creating the `System`.
-    Configuration flags can be used in equation strings, as long as they are numerical values.
-    To add config flags, use ::
+    Next, additional configuration flags can be added. Configuration flags for
+    models are load-time variables, specifying the behavior of a model. They can
+    be exported to an `andes.rc` file and automatically loaded when creating the
+    `System`. Configuration flags can be used in equation strings, as long as
+    they are numerical values. To add config flags, use ::
 
         self.config.add(OrderedDict((('pq2z', 1), )))
 
-    It is recommended to use `OrderedDict` instead of `dict`, although the syntax is verbose.
-    Note that booleans should be provided as integers (1, or 0), since `True` or `False` is interpreted as
-    a string when loaded from the `rc` file and will cause an error.
+    It is recommended to use `OrderedDict` instead of `dict`, although the
+    syntax is verbose. Note that booleans should be provided as integers (1 or
+    0), since `True` or `False` is interpreted as a string when loaded from the
+    `rc` file and will cause an error.
 
-    Next, it's time for variables and equations! The `PQ` class does not have internal variables itself.
-    It uses its `bus` parameter to fetch the corresponding `a` and `v` variables of buses.
-    Equation wise, it imposes an active power and a reactive power load equation.
+    Next, it's time for variables and equations! The `PQ` class does not have
+    internal variables itself. It uses its `bus` parameter to fetch the
+    corresponding `a` and `v` variables of buses. Equation wise, it imposes an
+    active power and a reactive power load equation.
 
     To define external variables from `Bus`, use ::
 
@@ -115,18 +123,20 @@ class Model:
             self.a.e_str = "u * p"
             self.v.e_str = "u * q"
 
-    where the `e_str` attribute is the equation string attribute. `u` is the connectivity status.
-    Any parameter, config, service or variables can be used in equation strings.
+    where the `e_str` attribute is the equation string attribute. `u` is the
+    connectivity status. Any parameter, config, service or variable can be used
+    in equation strings.
 
-    Three additional scalars can be used in equations:
-    - ``dae_t`` for the current simulation time can be used if the model has flag `tds`.
-    - ``sys_f`` for system frequency (from ``system.config.freq``).
-    - ``sys_mva`` for system base mva (from ``system.config.mva``).
+    Three additional scalars can be used in equations: - ``dae_t`` for the
+    current simulation time (can be used if the model has flag `tds`). -
+    ``sys_f`` for system frequency (from ``system.config.freq``). - ``sys_mva``
+    for system base mva (from ``system.config.mva``).
 
-    The above example is overly simplified. Our `PQ` model wants a feature to switch itself to
-    a constant impedance if the voltage is out of the range `(vmin, vmax)`.
-    To implement this, we need to introduce a discrete component called `Limiter`, which yields three arrays
-    of binary flags, `zi`, `zl`, and `zu` indicating in range, below lower limit, and above upper limit,
+    The above example is overly simplified. Our `PQ` model wants a feature to
+    switch itself to a constant impedance if the voltage is out of the range
+    `(vmin, vmax)`. To implement this, we need to introduce a discrete component
+    called `Limiter`, which yields three arrays of binary flags, `zi`, `zl`, and
+    `zu` indicating in-range, below lower-limit, and above upper-limit,
     respectively.
 
     First, create an attribute `vcmp` as a `Limiter` instance ::
@@ -134,8 +144,9 @@ class Model:
             self.vcmp = Limiter(u=self.v, lower=self.vmin, upper=self.vmax,
                                  enable=self.config.pq2z)
 
-    where `self.config.pq2z` is a flag to turn this feature on or off.
-    After this line, we can use `vcmp_zi`, `vcmp_zl`, and `vcmp_zu` in other equation strings. ::
+    where `self.config.pq2z` is a flag to turn this feature on or off. After
+    this line, we can use `vcmp_zi`, `vcmp_zl`, and `vcmp_zu` in other equation
+    strings. ::
 
             self.a.e_str = "u * (p0 * vcmp_zi + " \
                            "p0 * vcmp_zl * (v ** 2 / vmin ** 2) + " \
@@ -145,20 +156,17 @@ class Model:
                            "q0 * vcmp_zl * (v ** 2 / vmin ** 2) + "\
                            "q0 * vcmp_zu * (v ** 2 / vmax ** 2))"
 
-    Note that `PQ.a.e_str` can use the three variables from `vcmp` even before defining `PQ.vcmp`, as long as
-    `PQ.vcmp` is defined, because `vcmp_zi` is just a string literal in `e_str`.
+    Note that `PQ.a.e_str` can use the three variables from `vcmp` even before
+    defining `PQ.vcmp`, as long as `PQ.vcmp` is defined, because `vcmp_zi` is
+    just a string literal in `e_str`.
 
-    The two equations above implements a piecewise power injection equation. It selects the original power demand
-    if within range, and uses the calculated power when out of range.
-
-    Finally, to let ANDES pick up the model, the model name needs to be added to `models/__init__.py`.
-    Follow the examples in the `OrderedDict`, where the key is the file name, and the value is the class name.
-
-    Notes
-    -----
-    To modify parameters or services use ``set()``, which writes directly to the given attribute,
-    or ``alter()``, which converts parameters to system base like that for input data.
+    The two equations above implement a piece-wise power injection equation. It
+    selects the original power demand if within range, and uses the calculated
+    power when out of range.
 
+    Finally, to let ANDES pick up the model, the model name needs to be added to
+    `models/__init__.py`. Follow the examples in the `OrderedDict`, where the
+    key is the file name, and the value is the class name.
     """
 
     def __init__(self, system=None, config=None):
@@ -276,6 +284,7 @@ class Model:
 
         self.coeffs = dict()  # pu conversion coefficient storage
         self.bases = dict()   # base storage, such as Vn, Vb, Zn, Zb
+        self.debug_equations = list()  # variable names for debugging corresponding equation
 
     def _register_attribute(self, key, value):
         """
@@ -406,7 +415,7 @@ class Model:
 
     def _one_idx2uid(self, idx):
         """
-        Helper function for checking if an idx exist and
+        Helper function for checking if an idx exists and
         converting it to uid.
         """
 
@@ -468,7 +477,13 @@ class Model:
         """
         Set the value of an attribute of a model property.
 
-        Performs ``self.<src>.<attr>[idx] = value``.
+        Performs ``self.<src>.<attr>[idx] = value``. This method will not modify
+        the input values from the case file that have not been converted to the
+        system base. As a result, changes applied by this method will not affect
+        the dumped case file.
+
+        To alter parameters and reflect it in the case file, use :meth:`alter`
+        instead.
 
         Parameters
         ----------
@@ -493,11 +508,15 @@ class Model:
 
     def alter(self, src, idx, value):
         """
-        Alter input parameter or service values.
+        Alter values of input parameters or constant service.
 
-        If operates on a parameter, the input should be in the same
-        base as that in the input file. This function will convert
-        the new value to system-base per unit.
+        If the method operates on an input parameter, the new data should be in
+        the same base as that in the input file. This function will convert the
+        new value to per unit in the system base.
+
+        The values for storing the input data, i.e., the ``vin`` field of the
+        parameter, will be overwritten, thus the update will be reflected in the
+        dumped case file.
 
         Parameters
         ----------
@@ -524,8 +543,8 @@ class Model:
         ----------
         refresh : bool
             Refresh the values in the dictionary.
-            This is only used when the memory address of arrays changed.
-            After initialization, all array assignments are inplace.
+            This is only used when the memory addresses of arrays change.
+            After initialization, all array assignments are in place.
             To avoid overhead, refresh should not be used after initialization.
 
         Returns
@@ -669,7 +688,7 @@ class Model:
             if instance.has_check_var:
                 instance.check_var(dae_t=dae_t, niter=niter, err=err)
 
-    def l_check_eq(self, init=False, **kwargs):
+    def l_check_eq(self, init=False, niter=0, **kwargs):
         """
         Call the ``check_eq`` method of discrete components to update equation-dependent flags.
 
@@ -686,11 +705,12 @@ class Model:
                     instance.check_eq(allow_adjust=self.config.allow_adjust,
                                       adjust_lower=self.config.adjust_lower,
                                       adjust_upper=self.config.adjust_upper,
+                                      niter=0
                                       )
         else:
             for instance in self.discrete.values():
                 if instance.has_check_eq:
-                    instance.check_eq()
+                    instance.check_eq(niter=niter)
 
     def s_update(self):
         """
@@ -896,7 +916,7 @@ class Model:
         Notes
         -----
         In-place equations: added to the corresponding DAE array.
-        Non-inplace equations: in-place set to internal array to
+        Non-in-place equations: in-place set to internal array to
         overwrite old values (and avoid clearing).
         """
         if callable(self.calls.f):
@@ -1151,7 +1171,7 @@ class Model:
         Models with no back references will have internal variable addresses assigned but external addresses
         being empty.
 
-        For internal equations that has external variables, the row indices will be non-zeros, while the col
+        For internal equations that have external variables, the row indices will be non-zeros, while the col
         indices will be empty, which causes an error when updating Jacobians.
 
         Setting `self.in_use` to False when `len(back_ref_instance.v) == 0` avoids this error. See COI.
@@ -1244,7 +1264,9 @@ class Model:
         """
         Symbolic processing and code generation.
         """
-        logger.debug(f"Generating code for {self.class_name}")
+
+        logger.debug("Generating code for %s", self.class_name)
+
         self.calls.md5 = self.get_md5()
 
         self.syms.generate_symbols()
@@ -1320,7 +1342,7 @@ class Model:
         Warnings
         --------
         This feature is experimental and does not guarantee a speed up.
-        In fact, the program will likely end up slower due to compilation.
+        In fact, the program will likely end up being slower due to compilation.
         """
         if self.system.config.numba != 1:
             return
@@ -1380,14 +1402,6 @@ class Model:
 
         return f'{self.class_name} ({self.n} {dev_text}) at {hex(id(self))}'
 
-    def _log_init_debug(self, *args):
-        """
-        Helper function to log initialization debug message.
-        """
-
-        if self.system.options.get("init") is True:
-            logger.debug(*args)
-
     def init(self, routine):
         """
         Numerical initialization of a model.
@@ -1410,46 +1424,55 @@ class Model:
         else:
             do_init = getattr(self.flags, flag_name)
 
-        self._log_init_debug('%s has <%s> = %s', self.class_name, flag_name, do_init)
+        sys_debug = self.system.options.get("init")
+
+        logger.debug('========== %s has <%s> = %s ==========',
+                     self.class_name, flag_name, do_init)
 
         if do_init:
             kwargs = self.get_inputs(refresh=True)
 
-            self._log_init_debug('%s: initialization sequence:', self.class_name)
-            self._log_init_debug(' -> '.join([str(i) for i in self.calls.init_seq]))
-            self._log_init_debug("%s: assignment initialization phase begins", self.class_name)
+            logger.debug('Initialization sequence:')
+            seq_str = ' -> '.join([str(i) for i in self.calls.init_seq])
+            logger.debug('\n'.join(wrap(seq_str, 70)))
+            logger.debug("%s: assignment initialization phase begins", self.class_name)
 
             for idx, name in enumerate(self.calls.init_seq):
+                debug_flag = sys_debug or (name in self.debug_equations)
+
                 # single variable - do assignment
                 if isinstance(name, str):
-                    self._log_init_debug("%s: entering <%s> assignment init", self.class_name, name)
+                    _log_init_debug(debug_flag,
+                                    "%s: entering <%s> assignment init",
+                                    self.class_name, name)
+
                     instance = self.__dict__[name]
                     if instance.discrete is not None:
-                        self._log_init_debug("%s: evaluate discrete <%s>", name, instance.discrete)
+                        _log_init_debug(debug_flag, "%s: evaluate discrete <%s>", name, instance.discrete)
                         _eval_discrete(instance, self.config.allow_adjust,
                                        self.config.adjust_lower, self.config.adjust_upper)
 
                     if instance.v_str is not None:
                         arg_print = OrderedDict()
-                        if self.system.options.get("init"):
+                        if debug_flag:
                             for a, b in zip(self.calls.ia_args[name], self.ia_args[name]):
                                 arg_print[a] = b
 
                         if not instance.v_str_add:
                             # assignment is for most variable initialization
-                            self._log_init_debug("%s: new values will be assigned (=)", name)
+                            _log_init_debug(debug_flag, "%s: new values will be assigned (=)", name)
                             instance.v[:] = self.calls.ia[name](*self.ia_args[name])
 
                         else:
                             # in-place add initial values.
                             # Voltage compensators can set part of the `v` of exciters.
                             # Exciters will then set the bus voltage part.
-                            self._log_init_debug("%s: new values will be in-place added (+=)", name)
+                            _log_init_debug(debug_flag, "%s: new values will be in-place added (+=)", name)
                             instance.v[:] += self.calls.ia[name](*self.ia_args[name])
 
                         arg_print[name] = instance.v
 
-                        if self.system.options.get("init"):
+                        if debug_flag:
                             for key, val in arg_print.items():
                                 if isinstance(val, (int, float, np.floating, np.integer)) or \
                                         isinstance(val, np.ndarray) and val.ndim == 0:
@@ -1462,49 +1485,52 @@ class Model:
                                       header=["idx", *self.calls.ia_args[name], name],
                                       data=list(zip(self.idx.v, *arg_print.values())),
                                       )
-                            self._log_init_debug(tab.draw())
+                            _log_init_debug(debug_flag, tab.draw())
 
                     # single variable iterative solution
                     if name in self.calls.ii:
-                        self._log_init_debug("\n%s: entering <%s> iterative init", self.class_name,
-                                             pprint.pformat(name))
+                        _log_init_debug(debug_flag,
+                                        "\n%s: entering <%s> iterative init",
+                                        self.class_name,
+                                        pprint.pformat(name))
 
                         self.solve_iter(name, kwargs)
 
-                        self._log_init_debug("%s new values are %s", name, self.__dict__[name].v)
+                        _log_init_debug(debug_flag,
+                                        "%s new values are %s", name, self.__dict__[name].v)
 
                 # multiple variables, iterative
                 else:
-                    self._log_init_debug("\n%s: entering <%s> iterative init",
-                                         self.class_name, name)
+                    _log_init_debug(debug_flag, "\n%s: entering <%s> iterative init",
+                                    self.class_name, name)
 
                     for vv in name:
                         instance = self.__dict__[vv]
 
                         if instance.discrete is not None:
-                            self._log_init_debug("%s: evaluate discrete <%s>",
-                                                 name, instance.discrete)
+                            _log_init_debug(debug_flag, "%s: evaluate discrete <%s>",
+                                            name, instance.discrete)
 
                             _eval_discrete(instance, self.config.allow_adjust,
                                            self.config.adjust_lower, self.config.adjust_upper)
                         if instance.v_str is not None:
-                            self._log_init_debug("%s: v_str = %s", vv, instance.v_str)
+                            _log_init_debug(debug_flag, "%s: v_str = %s", vv, instance.v_str)
 
                             arg_print = OrderedDict()
-                            if self.system.options.get("init"):
+                            if debug_flag:
                                 for a, b in zip(self.calls.ia_args[vv], self.ia_args[vv]):
                                     arg_print[a] = b
-                                self._log_init_debug(pprint.pformat(arg_print))
+                                _log_init_debug(debug_flag, pprint.pformat(arg_print))
 
                             instance.v[:] = self.calls.ia[vv](*self.ia_args[vv])
 
-                    self._log_init_debug("\n%s: entering <%s> iterative init", self.class_name,
-                                         pprint.pformat(name))
+                    _log_init_debug(debug_flag, "\n%s: entering <%s> iterative init", self.class_name,
+                                    pprint.pformat(name))
                     self.solve_iter(name, kwargs)
 
                     for vv in name:
                         instance = self.__dict__[vv]
-                        self._log_init_debug("%s new values are %s", vv, instance.v)
+                        _log_init_debug(debug_flag, "%s new values are %s", vv, instance.v)
 
             # call custom variable initializer after generated init
             kwargs = self.get_inputs(refresh=True)
@@ -1601,6 +1627,16 @@ class Model:
         """
         pass
 
+    def register_debug_equation(self, var_name):
+        """
+        Helper function to register a variable for debugging the initialization.
+
+        This function needs to be called before calling ``TDS.init()``, and
+        logging level needs to be set to ``DEBUG``.
+        """
+
+        self.debug_equations.append(var_name)
+
 
 def _eval_discrete(instance, allow_adjust=True,
                    adjust_lower=False, adjust_upper=False):
@@ -1635,6 +1671,11 @@ def _eval_discrete(instance, allow_adjust=True,
                     adjust_upper=adjust_upper,
                     is_init=True,
                     )
+        d.check_eq(allow_adjust=allow_adjust,
+                   adjust_lower=adjust_lower,
+                   adjust_upper=adjust_upper,
+                   is_init=True,
+                   )
 
 
 def to_jit(func: Union[Callable, None],
@@ -1657,3 +1698,12 @@ def to_jit(func: Union[Callable, None],
                          )
 
     return func
+
+
+def _log_init_debug(debug_flag, *args):
+    """
+    Helper function to log initialization debug message.
+    """
+
+    if debug_flag is True:
+        logger.debug(*args)
