@@ -8,6 +8,7 @@
 #  File name: service.py
 #  Last modified: 8/16/20, 7:28 PM
 
+from functools import partial
 import logging
 from collections import OrderedDict
 from typing import Callable, Optional, Type, Union
@@ -426,8 +427,8 @@ class ExtendedEvent(VarService):
                 self.n_ext += len(ending)
 
                 # TODO: insert extended event end times to a model-level list
-                logger.debug("Extended Event ending time set at t=%.4f sec.",
-                             final_times)
+                logger.debug("Extended Event ending time set at t=%s sec.",
+                             str(final_times))
 
         # final time of the extended event
         if self.n_ext and np.any(self.t_final <= dae_t):
@@ -665,16 +666,30 @@ class DataSelect(BaseService):
 
 class DeviceFinder(BaseService):
     """
-    Service for finding indices of optionally linked devices.
+    Service for finding ``idx`` of devices which are linked to the given
+    devices.
 
-    If not provided, `DeviceFinder` will add devices at the beginning of
-    `System.setup`.
+    The ``auto_find`` parameter controls if the device idx should be
+    automatically looked up. The ``auto_add`` parameter controls if the device
+    will be automatically added. The two parameters are not exclusive. One can
+    skip finding the device but automatically adding it.
+
+    If ``auto_find`` is ``True`` and the ``idx`` is None, DeviceFinder will look
+    up for the device. If not found and ``auto_add`` is ``True``, DevFinder will
+    then automatically add the devices. The ``idx`` of the devices that are
+    found or added will be stored to the DeviceFinder instance, so that
+    `DeviceFinder` can be used like any `IdxParam`.
+
+    Adding new devices are called at the beginning of
+    :py:meth:`andes.system.System.setup`.
 
     Examples
     --------
-    IEEEST stabilizer takes an optional `busf` (IdxParam) for specifying the
-    connected BusFreq, which is needed for mode 6. To avoid reimplementing
-    `BusFreq` within IEEEST, one can do
+
+    The IEEEST stabilizer takes an optional parameter ``busf`` of the type
+    `IdxParam` for specifying the connected bus frequency measurement device,
+    which is needed for mode 6. To avoid reimplementing `BusFreq` within IEEEST,
+    one can do
 
     .. code-block :: python
 
@@ -682,30 +697,49 @@ class DeviceFinder(BaseService):
                                     link=self.buss, idx_name='bus',
                                     default_model='BusFreq')
 
-    where ``self.busf`` is the optional input, ``self.buss`` is the bus indices
-    that ``busf`` should measure, and ``idx_name`` is the name of a BusFreq
-    parameter through which the measured bus indices are specified. For each
-    ``None`` values in ``self.busf``, a ``BusFreq`` device will be created to
-    measure the corresponding bus in ``self.buss``.
+    where ``self.busf`` is for the optional parameter for the ``idx`` of bus
+    frequency estimation devices (e.g., `BusFreq`), ``self.buss`` is for the
+    ``idx`` of buses that ``self.busf`` devices should measure, and ``idx_name``
+    is the name of the BusFreq parameter through which the indices of measured
+    buses are given.
 
-    That is, ``BusFreq.[idx_name].v = [link]``. ``DeviceFinder`` will find or
-    create ``BusFreq`` devices so that the returned list of ``BusFreq`` indices
-    are connected to `self.buss`, respectively.
+    For each ``None`` or invalid values in ``self.busf``, a `BusFreq` device
+    will be created with its ``bus`` set to the corresponding value in
+    ``self.buss``. That is, ``BusFreq.[idx_name].v = [link]``.
+
+    At the end, the `DeviceFinder` instance will contain the list of ``BusFreq``
+    that are are connected to `self.buss`, respectively.
+
+    In the case of any valid value in ``self.busf``, that is, the value is an
+    existing ``BusFreq`` device, `DeviceFinder` will return it as is without
+    checking if the ``BusFreq`` device actually measures the bus specified by
+    ``self.buss``. It allows to use the measurement at a different location, but
+    the user have to perform the data consistency check.
     """
 
-    def __init__(self, u, link, idx_name, default_model,
-                 name=None, tex_name=None, info=None):
+    def __init__(self, u, link,
+                 idx_name: str,
+                 default_model: str,
+                 auto_find: Optional[bool] = True,
+                 auto_add: Optional[bool] = True,
+                 name: Optional[str] = None,
+                 tex_name: Optional[str] = None,
+                 info: Optional[str] = None):
+
         super().__init__(name=name, tex_name=tex_name, info=info)
 
         self.u = u
         self.model = u.model
         self.idx_name = idx_name
         self.default_model = default_model
+        self.auto_find = auto_find
+        self.auto_add = auto_add
 
         if self.model is None:
             raise ValueError(f'{u.owner.class_name}.{u.name} must contain "model".')
 
         self.link = link
+        self.v = None
 
     def find_or_add(self, system):
         """
@@ -717,6 +751,11 @@ class DeviceFinder(BaseService):
         Devices previously added in this function can be used later without duplication.
         """
 
+        # make a copy of indices from `u`
+        self.v = list(self.u.v)
+
+        # determine model or group
+
         if self.model in system.models:
             is_model = True
         elif self.model in system.groups:
@@ -730,18 +769,33 @@ class DeviceFinder(BaseService):
         added = False
 
         for ii, link_to in enumerate(self.link.v):
-            idx = mdl.find_idx(self.idx_name, (link_to, ), allow_none=True, default=None)[0]
 
-            if idx is None:
-                new_idx = system.add(add_to_model, {self.idx_name: link_to})
-                self.u.v[ii] = new_idx
+            idx = self.v[ii]
+            valid_idx = None
+
+            # check if ``idx`` is valid
+            if idx is not None:
+                valid_idx = mdl.find_idx('idx', (idx, ), allow_none=True, default=None)[0]
+                if valid_idx == idx:
+                    continue
+
+                logger.warning("%s.%s: <%s> is not found.",
+                               self.u.owner.class_name, self.u.name, idx)
+
+            if (not valid_idx) and self.auto_find:
+                idx = mdl.find_idx(self.idx_name, (link_to, ), allow_none=True, default=None)[0]
+                if idx is not None:
+                    self.v[ii] = idx
+                    continue
+
+            if (not valid_idx) and self.auto_add:
+                idx = system.add(add_to_model, {self.idx_name: link_to})
+                self.v[ii] = idx
 
                 logger.info(f"{self.owner.class_name} <{self.owner.idx.v[ii]}> "
-                            f"added {add_to_model} <{new_idx}> "
+                            f"added {add_to_model} <{idx}> "
                             f"linked to {self.idx_name} <{link_to}>")
                 added = True
-            else:
-                self.u.v[ii] = idx
 
         if added:
             mdl = system.models[add_to_model]
@@ -749,10 +803,6 @@ class DeviceFinder(BaseService):
             mdl.refresh_inputs()
             system.link_ext_param({mdl.name: mdl})
 
-    @property
-    def v(self):
-        return self.u.v
-
 
 class OperationService(BaseService):
     """
@@ -1060,15 +1110,19 @@ class NumSelect(OperationService):
 
     NumSelect works with internal and external parameters.
 
+    Any values equal to ``np.nan`` will always be ignored. If one needs to
+    ignore values based on additional conditions, pass it through
+    ``ignore_cond``. For example, to ignore zero values, use ``ignore_cond =
+    partial(np.equal, 0)``.
+
     Examples
     --------
     One use case is to allow an optional turbine rating. One can do
 
     .. code:: python
 
-        self.Tn = NumParam(default=None)
-        self.Sg = ExtParam(...)
-        self.Sn = DataSelect(Tn, Sg)
+        self.Tn = NumParam(default=None) self.Sg = ExtParam(...) self.Sn =
+        DataSelect(Tn, Sg)
 
     """
 
@@ -1078,19 +1132,31 @@ class NumSelect(OperationService):
                  name: Optional[str] = None,
                  tex_name: Optional[str] = None,
                  info: Optional[str] = None,
+                 ignore_cond: Optional[Callable] = partial(np.equal, 0),
                  ):
         super().__init__(name=name, tex_name=tex_name, info=info)
         self.optional = optional
         self.fallback = fallback
+        self.ignore_cond = ignore_cond
         self._v = None
 
     @property
     def v(self):
         if self._v is None:
-            self._v = [v1 if not np.isnan(v1)
-                       else v2
-                       for v1, v2 in zip(self.optional.v, self.fallback.v)]
+            self._v = list()
 
+            for v1, v2 in zip(self.optional.v, self.fallback.v):
+                if np.isnan(v1):
+                    self._v.append(v2)
+
+                elif self.ignore_cond is not None and \
+                        self.ignore_cond(v1):
+                    self._v.append(v2)
+
+                else:
+                    self._v.append(v1)
+
+            # when done, convert `self._v` to an array
             self._v = np.array(self._v)
 
         return self._v
