@@ -49,10 +49,15 @@ class TDS(BaseRoutine):
                                      ('qrt', 0),
                                      ('kqrt', 1.0),
                                      ('store_z', 0),
-                                     ('store_f', 0.0),
-                                     ('store_h', 0.0),
-                                     ('store_i', 0.0),
-                                     ('no_tqdm', 0.0),
+                                     ('store_f', 0),
+                                     ('store_h', 0),
+                                     ('store_i', 0),
+                                     ('limit_store', 0),
+                                     ('max_store', 900),
+                                     ('save_every', 1),
+                                     ('save_mode', 'auto'),
+                                     ('no_tqdm', 0),
+                                     ('chatter_iter', 4),
                                      )))
         self.config.add_extra("_help",
                               method='DAE solution method',
@@ -77,7 +82,12 @@ class TDS(BaseRoutine):
                               store_f='store RHS of diff. equations',
                               store_h='store RHS of external diff. equations',
                               store_i='store RHS of external algeb. equations',
+                              limit_store='limit in-memory timeseries storage',
+                              max_store='maximum steps of data stored in memory before offloading',
+                              save_every='save results for one step every "save_every" steps',
+                              save_mode='automatically or manually save output data when done',
                               no_tqdm='disable tqdm progressbar and outputs',
+                              chatter_iter='minimum iterations to detect chattering',
                               )
         self.config.add_extra("_alt",
                               method=tuple(method_map.keys()),
@@ -101,7 +111,12 @@ class TDS(BaseRoutine):
                               store_f=(0, 1),
                               store_h=(0, 1),
                               store_i=(0, 1),
+                              limit_store=(0, 1),
+                              max_store='positive integer',
+                              save_every='integer',
+                              save_mode=('auto', 'manual'),
                               no_tqdm=(0, 1),
+                              chatter_iter='int>=4',
                               )
 
         # overwrite `tf` from command line
@@ -129,6 +144,7 @@ class TDS(BaseRoutine):
 
         # internal status
         self.converged = False
+        self.chatter = False
         self.last_converged = False   # True if the previous step converged
         self.busted = False           # True if in a non-recoverable error state
         self.err_msg = ''
@@ -168,6 +184,7 @@ class TDS(BaseRoutine):
             The initial values of xy.
 
         """
+
         t0, _ = elapsed()
         system = self.system
 
@@ -186,6 +203,7 @@ class TDS(BaseRoutine):
         #   to the new array after extending `dae.y`.
         system.set_address(models=system.exist.pflow_tds)
         system.set_dae_names(models=system.exist.tds)
+        system.set_output_subidx(models=system.exist.pflow_tds)
 
         system.dae.clear_ts()
         system.store_sparse_pattern(models=system.exist.pflow_tds)
@@ -265,6 +283,7 @@ class TDS(BaseRoutine):
         -------
         None
         """
+
         out = list()
         out.append('')
         out.append('-> Time Domain Simulation Summary:')
@@ -344,9 +363,15 @@ class TDS(BaseRoutine):
         self.qrt_start = time.time()
         self.headroom = 0.0
 
+        # write variable list file at the beginning
+        if not system.files.no_output:
+            system.dae.write_lst(self.system.files.lst)
+
         t0, _ = elapsed()
 
         while (system.dae.t - self.h < self.config.tf) and (not self.busted):
+
+            # call perturbation file if specified
             if self.callpert is not None:
                 self.callpert(dae.t, system)
 
@@ -358,21 +383,40 @@ class TDS(BaseRoutine):
                 step_status = self._csv_step()
 
             # record number of iterations and success flag
-            if system.config.call_stats:
+            if system.config.save_stats:
                 self.call_stats.append((system.dae.t.tolist(), self.niter, step_status))
 
             if step_status:
-                dae.store()
+                if config.save_every != 0:
+                    if config.save_every == 1:
+                        dae.store()
+                    else:
+                        if dae.kcount % config.save_every == 0:
+                            dae.store()
+
+                # offload if exceeds `max_store`
+                if self.config.limit_store and len(dae.ts._ys) >= self.config.max_store:
+
+                    # write to file if enabled
+                    if not system.files.no_output:
+                        self.save_output()
+                        logger.info("Offload data from memory to file for t=%.2f - %.2f sec",
+                                    dae.ts.t[0], dae.ts.t[-1])
+
+                    # clear storage in memory anyway
+                    dae.ts.reset()
 
                 self.streaming_step()
+
                 if self.check_criteria() is False:
-                    self.err_msg = 'Violated stability criteria.'
+                    self.err_msg = 'Violated stability criteria. To turn off, set [TDS].criteria = 0.'
                     self.busted = True
 
                 # check if the next step is critical time
                 self.do_switch()
                 self.calc_h()
                 dae.t += self.h
+                dae.kcount += 1
 
                 # show progress in percentage
                 perc = max(min((dae.t - config.t0) / (config.tf - config.t0) * 100, 100), 0)
@@ -388,8 +432,10 @@ class TDS(BaseRoutine):
                     rt_end = self.qrt_start + self.h * config.kqrt
 
                     # if the ending time has passed
-                    if time.time() - rt_end > 0:
-                        logger.debug('Simulation over-run at t=%4.4g s.', dae.t)
+                    t_overrun = time.time() - rt_end
+                    if t_overrun > 0:
+                        logger.debug('Simulation over-run for %4.4g msec at t=%4.4g s.',
+                                     1000 * t_overrun, dae.t)
                     else:
                         self.headroom += (rt_end - time.time())
 
@@ -415,20 +461,30 @@ class TDS(BaseRoutine):
         else:
             system.exit_code += 1
 
+        # removed `pbar` so that System object can be serialized
         self.pbar.close()
-        self.pbar = None  # removed `pbar` so that System object can be serialized
+        self.pbar = None
 
         t1, s1 = elapsed(t0)
         self.exec_time = t1 - t0
-        logger.info('Simulation completed in %s.', s1)
+        logger.info('Simulation to t=%.2f sec completed in %s.', config.tf, s1)
 
         if config.qrt:
             logger.debug('QRT headroom time: %.4g s.', self.headroom)
 
-        # need to unpack data in case of resumed simulations.
-        system.dae.ts.unpack()
-        if not system.files.no_output:
+        # in case of resumed simulations,
+        # manually unpack data to update arrays in `dae.ts`
+        # disable warning in case data has just been dumped
+        system.dae.ts.unpack(warn_empty=False)
+
+        if (not system.files.no_output) and (config.save_mode == 'auto'):
+            t0, _ = elapsed()
             self.save_output()
+            _, s1 = elapsed(t0)
+
+            np_file = self.system.files.npz
+            logger.info('Outputs to "%s" and "%s".', self.system.files.lst, np_file)
+            logger.info('Outputs written in %s.', s1)
 
         # end data streaming
         if system.config.dime_enabled:
@@ -456,6 +512,7 @@ class TDS(BaseRoutine):
         """
         Fetch data for the next step from ``data_csv``.
         """
+
         system = self.system
         if self.data_csv is not None:
             system.dae.x[:] = self.data_csv[self.k_csv, 1:system.dae.n + 1]
@@ -487,6 +544,7 @@ class TDS(BaseRoutine):
         float
             computed time step size stored in ``self.h``
         """
+
         system = self.system
         config = self.config
 
@@ -512,6 +570,14 @@ class TDS(BaseRoutine):
                 # for converged cases, set step size back to the initial `config.tstep`
                 if config.fixt:
                     self.deltat = min(config.tstep, self.deltat)
+
+                if self.chatter is True:
+                    # one can do something such as increasing the step size, but
+                    # stopping chattering is not guaranteed
+
+                    # remember of unset the `chatter` flag
+                    self.chatter = False
+
             else:
                 self.deltat *= 0.9
                 if self.deltat < self.deltatmin:
@@ -543,6 +609,7 @@ class TDS(BaseRoutine):
         """
         Compute the first time step and save to ``self.h``.
         """
+
         system = self.system
         config = self.config
 
@@ -598,14 +665,19 @@ class TDS(BaseRoutine):
         """
         Manually load a plotter into ``TDS.plotter``.
         """
+
         from andes.plot import TDSData  # NOQA
         self.plotter = TDSData(mode='memory', dae=self.system.dae)
         self.plt = self.plotter
 
     def test_init(self):
         """
-        Update f and g to see if initialization is successful.
+        Test if the TDS initialization is successful.
+
+        This function update ``dae.f`` and ``dae.g`` and checks if the residuals
+        are zeros.
         """
+
         system = self.system
         # fg_update is called in TDS.init()
         system.j_update(models=system.exist.pflow_tds)
@@ -662,35 +734,27 @@ class TDS(BaseRoutine):
         ----------
         npz : bool
             True to save in npz format; False to save in npy format.
+
         Returns
         -------
         bool
             True if files are written. False otherwise.
         """
 
-        t0, _ = elapsed()
-
-        self.system.dae.write_lst(self.system.files.lst)
-
         if npz is True:
-            np_file = self.system.files.npz
             self.system.dae.write_npz(self.system.files.npz)
         else:
-            np_file = self.system.files.npy
             self.system.dae.write_npy(self.system.files.npy)
 
-        _, s1 = elapsed(t0)
+        self.system.dae.ts.idx_ptr = len(self.system.dae.ts.t)
 
-        logger.info('Outputs to "%s" and "%s".', self.system.files.lst, np_file)
-        logger.info('Outputs written in %s.', s1)
         return True
 
     def do_switch(self):
         """
         Checks if is an event time and perform switch if true.
-
-        Time is approximated with a tolerance of 1e-8.
         """
+
         ret = False
         system = self.system
 
@@ -757,7 +821,7 @@ class TDS(BaseRoutine):
         # 12/08/2020: Moved `l_update_eq` to before `g_update`
         #   because some algebraic variables depend on pegged states.
         system.f_update(models=models)
-        system.l_update_eq(models=models, init=init)
+        system.l_update_eq(models=models, init=init, niter=self.niter)
 
         system.g_update(models=models)
         system.fg_to_dae()
@@ -790,6 +854,7 @@ class TDS(BaseRoutine):
         """
         Load perturbation files to ``self.callpert``.
         """
+
         system = self.system
         if not system.files.pert:
             return False
@@ -812,7 +877,13 @@ class TDS(BaseRoutine):
     def _load_csv(self, csv_file):
         """
         Load simulation data from CSV file and return a numpy array.
+
+        Parameters
+        ----------
+        csv_file : str
+            Path to the CSV file.
         """
+
         if csv_file is None:
             return None
 
@@ -850,8 +921,8 @@ class TDS(BaseRoutine):
         logger.debug('--> Iteration Number: niter = %d', self.niter)
         logger.debug('Max. algebraic equation mismatch:')
         logger.debug('  <%s> [y_idx=%d]', self.system.dae.y_name[y_idx], y_idx)
-        logger.debug('  Variable value = %.4f', self.system.dae.y[y_idx])
-        logger.debug('  Mismatch value = %.4f', self.system.dae.g[y_idx])
+        logger.debug('  Variable value = %.8f', self.system.dae.y[y_idx])
+        logger.debug('  Mismatch value = %.8f', self.system.dae.g[y_idx])
 
         assoc_vars = self.system.dae.gy[y_idx, :]
         vars_idx = np.where(np.ravel(matrix(assoc_vars)))[0]
@@ -965,7 +1036,9 @@ class TDS(BaseRoutine):
         """
         Set DAE solution method.
 
-        name : str, optional, default: trapezoid
+        Parameters
+        ----------
+        name : str, optional, default: 'trapezoid'
             DAE solver name
         """
 
