@@ -62,10 +62,13 @@ class System:
 
     Parameters
     ----------
-    no_undill : bool, optional
+    no_undill : bool, optional, default=False
         True to disable the call to ``System.undill()`` at the end of object creation.
         False by default.
 
+    autogen_stale : bool, optional, default=True
+        True to automatically generate code for stale models.
+
     Notes
     -----
     System stores model and routine instances as attributes.
@@ -98,6 +101,7 @@ class System:
                  default_config: Optional[bool] = False,
                  options: Optional[Dict] = None,
                  no_undill: Optional[bool] = False,
+                 autogen_stale: Optional[bool] = True,
                  **kwargs
                  ):
         self.name = name
@@ -144,7 +148,7 @@ class System:
                                      ('numba_parallel', 0),
                                      ('numba_nopython', 0),
                                      ('yapf_pycode', 0),
-                                     ('call_stats', 0),
+                                     ('save_stats', 0),
                                      ('np_divide', 'warn'),
                                      ('np_invalid', 'warn'),
                                      ('pickle_path', get_pkl_path())
@@ -161,7 +165,7 @@ class System:
                               numba_parallel='enable parallel for numba.jit',
                               numba_nopython='nopython mode for numba',
                               yapf_pycode='format generated code with yapf',
-                              call_stats='store statistics of function calls',
+                              save_stats='store statistics of function calls',
                               np_divide='treatment for division by zero',
                               np_invalid='treatment for invalid floating-point ops.',
                               pickle_path='path models should be (un)dilled to/from',
@@ -177,7 +181,7 @@ class System:
                               numba_parallel=(0, 1),
                               numba_nopython=(0, 1),
                               yapf_pycode=(0, 1),
-                              call_stats=(0, 1),
+                              save_stats=(0, 1),
                               np_divide={'ignore', 'warn', 'raise', 'call', 'print', 'log'},
                               np_invalid={'ignore', 'warn', 'raise', 'call', 'print', 'log'},
                               )
@@ -207,7 +211,7 @@ class System:
         self.is_setup = False        # if system has been setup
 
         if not no_undill:
-            self.undill()
+            self.undill(autogen_stale=autogen_stale)
 
     def _set_numpy(self):
         """
@@ -240,20 +244,31 @@ class System:
             newobj = True
 
         for item in config_option:
-            if item.count('=') != 1 or item.count('.') != 1:
-                logger.error("Invalid config_option option: {}".format(item))
-                continue
+
+            # check the validity of the config field
+            # each field follows the format `SECTION.FIELD = VALUE`
+
+            if item.count('=') != 1:
+                raise ValueError('config_option "{}" must be an assignment expression'.format(item))
 
             field, value = item.split("=")
+
+            if field.count('.') != 1:
+                raise ValueError('config_option left-hand side "{}" must use format SECTION.FIELD'.format(field))
+
             section, key = field.split(".")
 
+            section = section.strip()
+            key = key.strip()
+            value = value.strip()
+
             if not newobj:
                 self._config_object.set(section, key, value)
-                logger.debug("Config option set: {}={}".format(field, value))
+                logger.debug("Existing config option set: %s.%s=%s", section, key, value)
             else:
                 self._config_object.add_section(section)
                 self._config_object.set(section, key, value)
-                logger.debug("Config option added: [{}] {}={}".format(section, field, value))
+                logger.debug("New config option added: %s.%s=%s", section, key, value)
 
     def reload(self, case, **kwargs):
         """
@@ -947,7 +962,7 @@ class System:
             mdl.coeffs = coeffs
             mdl.bases = {'Sn': Sn, 'Sb': Sb, 'Vn': Vn, 'Vb': Vb, 'Zn': Zn, 'Zb': Zb}
 
-    def l_update_var(self, models: OrderedDict, niter=None, err=None):
+    def l_update_var(self, models: OrderedDict, niter=0, err=None):
         """
         Update variable-based limiter discrete states by calling ``l_update_var`` of models.
 
@@ -956,14 +971,14 @@ class System:
         self.call_models('l_update_var', models,
                          dae_t=self.dae.t, niter=niter, err=err)
 
-    def l_update_eq(self, models:  OrderedDict, init=False):
+    def l_update_eq(self, models:  OrderedDict, init=False, niter=0):
         """
         Update equation-dependent limiter discrete components by calling ``l_check_eq`` of models.
         Force set equations after evaluating equations.
 
         This function is must be called after differential equation updates.
         """
-        self.call_models('l_check_eq', models, init=init)
+        self.call_models('l_check_eq', models, init=init, niter=niter)
 
     def s_update_var(self, models: OrderedDict):
         """
@@ -1188,10 +1203,25 @@ class System:
         # collect from-bus and to-bus indices
         fr, to, u = list(), list(), list()
 
+        # TODO: generalize it to all serial devices
         # collect from Line
         fr.extend(self.Line.a1.a.tolist())
         to.extend(self.Line.a2.a.tolist())
         u.extend(self.Line.u.v.tolist())
+
+        # collect from Fortescue
+        fr.extend(self.Fortescue.a.a.tolist())
+        to.extend(self.Fortescue.aa.a.tolist())
+        u.extend(self.Fortescue.u.v.tolist())
+
+        fr.extend(self.Fortescue.a.a.tolist())
+        to.extend(self.Fortescue.ab.a.tolist())
+        u.extend(self.Fortescue.u.v.tolist())
+
+        fr.extend(self.Fortescue.a.a.tolist())
+        to.extend(self.Fortescue.ac.a.tolist())
+        u.extend(self.Fortescue.u.v.tolist())
+
         os = [0] * len(u)
 
         # find islanded buses
@@ -1490,21 +1520,40 @@ class System:
             logger.debug("Dumping calls to calls.pkl is not supported with NumPy 1.2+")
             logger.debug("ANDES is fully functional with generated Python code.")
 
-    def undill(self):
+    def undill(self, autogen_stale=True):
         """
-        Deserialize the function calls from ``~/.andes/calls.pkl`` with ``dill``.
+        Reload generated function functions, from either the
+        ``$HOME/.andes/pycode`` folder or the ``$HOME/.andes/calls.pkl`` file.
 
-        If no change is made to models, future calls to ``prepare()`` can be replaced with ``undill()`` for
-        acceleration.
+        If no change is made to models, future calls to ``prepare()`` can be
+        replaced with ``undill()`` for acceleration.
+
+        Parameters
+        ----------
+        autogen_stale: bool
+            True to automatically call code generation if stale code is
+            detected. Regardless of this option, codegen is trigger if importing
+            existing code fails.
         """
 
         # load equations and jacobian from saved code
         loaded = self._load_calls()
+
         stale_models = self._find_stale_models()
 
         if loaded is False:
             self.prepare(quick=True, incremental=False)
             loaded = True
+        elif autogen_stale is False:
+            # NOTE: incremental code generation may be triggered due to Python
+            # not invalidating ``.pyc`` caches. If multiprocessing is being
+            # used, code generation will cause nested multiprocessing, which is
+            # not allowed.
+            # The flag ``autogen_stale=False`` is used to prevent nested codegen
+            # and is intended to be used only in multiprocessing.
+            logger.info("Generated code for <%s> is stale.", ', '.join(stale_models.keys()))
+            logger.info("Automatic code re-generation manually skipped")
+            loaded = True
         elif len(stale_models) > 0:
             logger.info("Generated code for <%s> is stale.", ', '.join(stale_models.keys()))
             self.prepare(quick=True, incremental=True, models=stale_models)
@@ -1714,7 +1763,7 @@ class System:
         for name, mdl in models.items():
             ret[name] = getattr(mdl, method)(*args, **kwargs)
 
-            if self.config.call_stats:
+            if self.config.save_stats:
                 if method not in self.call_stats[name]:
                     self.call_stats[name][method] = 1
                 else:
@@ -1931,7 +1980,7 @@ class System:
         """
         Invoke the actions associated with switch times.
 
-        Switch actions will be disabled if `flat=True` is passed to system.
+        This function will not be called if ``flat=True`` is passed to system.
         """
         for instance in models.values():
             instance.switch_action(self.dae.t)
@@ -2148,6 +2197,64 @@ class System:
 
         return True
 
+    def set_output_subidx(self, models):
+        """
+        Process :py:class:`andes.models.misc.Output` data and store the
+        sub-indices into ``dae.xy``.
+
+        Parameters
+        ----------
+        models : OrderedDict
+            Models currently in use for the routine
+        """
+
+        export_vars = dict(x=list(), y=list())  # indices of export x and y
+
+        for model, var, dev in zip(self.Output.model.v,
+                                   self.Output.varname.v,
+                                   self.Output.dev.v):
+
+            # check validity of model name
+            if model not in models:
+                logger.info("Output model <%s> invalid or contains no device. Skipped.",
+                            model)
+                continue
+            mdl_instance = models[model]
+            mdl_all_vars = mdl_instance.cache.all_vars
+
+            # check validity of var name
+            if var is not None and (var not in mdl_all_vars):
+                logger.info("Output model <%s> contains no variable <%s>. Skipped.",
+                            model, var)
+                continue
+
+            # check validity of dev idx
+            if (dev is not None) and (dev not in mdl_instance.idx.v):
+                logger.info("Output model <%s> contains no device <%s>. Skipped.",
+                            model, dev)
+                continue
+
+            # TODO: dev-based indexing is not fully supported
+            # for multi-index variables, such as those in COI.
+
+            if var is None:
+                for item in mdl_all_vars.values():
+                    if dev is None:
+                        export_vars[item.v_code].extend(item.a)
+                    else:
+                        uid = mdl_instance.idx2uid(dev)
+                        export_vars[item.v_code].append(item.a[uid])
+            else:  # with variable name
+                item = mdl_all_vars[var]
+                if dev is None:
+                    export_vars[item.v_code].extend(item.a)
+                else:  # with exact index
+                    uid = mdl_instance.idx2uid(dev)
+                    export_vars[item.v_code].append(item.a[uid])
+
+        self.Output.xidx = sorted(np.unique(export_vars['x']))
+        self.Output.yidx = sorted(np.unique(export_vars['y']))
+
 
 def load_pycode_from_path(pycode_path):
     """
