@@ -214,8 +214,9 @@ def get_pkl_path():
 
 def get_dot_andes_path():
     """
-    Return the path to ``<HomeDir>/.andes``
+    Return the path to ``$HOME/.andes``
     """
+
     return os.path.join(str(pathlib.Path.home()), '.andes')
 
 
@@ -230,15 +231,16 @@ def get_log_dir():
     str
         The path to the temporary logging directory
     """
-    tempdir = os.path.join(tempfile.gettempdir(), 'andes')
-    if not os.path.exists(tempdir):
-        os.mkdir(tempdir)
-
+    tempdir = tempfile.gettempdir()
     path = tempfile.mkdtemp(prefix='andes-', dir=tempdir)
     return path
 
 
 def confirm_overwrite(outfile, overwrite=None):
+    """
+    Confirm overwriting a file.
+    """
+
     try:
         if os.path.isfile(outfile):
             if overwrite is None:
